@@ -5,6 +5,7 @@
  * Copyright (C) 2004-2005, Mathias Hasselmann <mathias@taschenorakel.de>
  * Copyright (C) 2005, Daniel Beardsmore <uilleann@users.sf.net>
  * Copyright (C) 2005, Björn Nilsson <BNI on irc.freenode.net>
+ * Copyright (C) 2005, Anthony Sofocleous <itchysoft_ant@users.sf.net>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -40,6 +41,7 @@
 #include <plugin.h>
 #include <pluginpref.h>
 #include <prefs.h>
+#include <util.h>
 #include <version.h>
 
 /* Gettext Stubs */
@@ -55,11 +57,19 @@
 
 #define IRC_PLUGIN_ID "prpl-irc"
 
+/* Copied from SECS_BEFORE_RESENDING_AUTORESPONSE in src/server.c */
+#define AUTO_RESPONSE_INTERVAL 600
+
+#define DOMAIN_SUFFIX_FREENODE ".freenode.net"
 #define DOMAIN_SUFFIX_GAMESURGE ".gamesurge.net"
 #define DOMAIN_SUFFIX_JEUX ".jeux.fr"
 #define DOMAIN_SUFFIX_QUAKENET ".quakenet.org"
 #define DOMAIN_SUFFIX_UNDERNET ".undernet.org"
 
+#define MESSAGE_CHANSERV_ACCESS_LIST_ADD "You have been added to the access list for "
+/* Omit the first character of the portion after the channel name. */
+#define MESSAGE_CHANSERV_ACCESS_LIST_ADD_WITH_LEVEL "with level ["
+#define MESSAGE_CHANSERV_ACCESS_LIST_DEL "You have been deleted from the access list for ["
 #define MESSAGE_CHANSERV_NO_OP_ACCESS "You do not have channel operator access to"
 #define MESSAGE_FRENCH_ADVERTISEMENT_START "<B>Avertissement</B> : Le pseudo <B>"
 #define MESSAGE_FRENCH_ADVERTISEMENT_END "&lt;votre pass&gt;"
@@ -73,6 +83,9 @@
 #define MESSAGE_INVITED " invited "
 #define MESSAGE_LOGIN_CONNECTION_COUNT "Highest connection count"
 #define MESSAGE_MEMOSERV_NO_NEW_MEMOS "You have no new memos"
+#define MESSAGE_MODE_NOTICE_PREFIX "mode (+"
+#define MESSAGE_MODE_NOTICE_SUFFIX " ) by "
+#define MESSAGE_NICKSERV_CLOAKED " set your hostname to"
 #define MESSAGE_NICKSERV_ID_FAILURE "Password Incorrect"
 #define MESSAGE_NICKSERV_IDENTIFIED "Password accepted - you are now recognized"
 #define MESSAGE_QUAKENET_Q_CRUFT \
@@ -96,6 +109,7 @@
 #define NICK_FREENODE_CONNECT "freenode-connect"
 #define NICK_GAMESURGE_AUTHSERV "AuthServ"
 #define NICK_GAMESURGE_AUTHSERV_SERVICE NICK_GAMESURGE_AUTHSERV "@Services.GameSurge.net"
+#define NICK_GAMESURGE_GLOBAL "Global"
 #define NICK_JEUX_Z "Z"
 #define NICK_JEUX_WELCOME "[Welcome]"
 #define NICK_MEMOSERV "MemoServ"
@@ -179,21 +193,12 @@ static GaimPluginInfo plugin_info =
 
 	/** description */
 	N_(
-		"- Transparent nickname authentication with a variety of services.\n"
-		"\t* Ability to kill ghosted usernames with NickServ\n"
-		"- Display of ChanServ and L channel join messages in chat\n"
-		"- Suppression of MemoServ message for no new memos\n"
-		"- Suppression of various useless system messages\n"
-		"- Transparent authentication as IRC op\n"
+		"- Transparent authentication with a variety of services.\n"
+		"- Suppression of various useless messages"
 	),
 
 	/** author */
-	"Richard Laager <rlaager@users.sf.net>"
-	"\n"
-	"\n\t\t\tQuakeNet support written by:"
-	"\n\t\t\t\tMathias Hasselmann"
-	"\n\t\t\t\t\t<mathias@taschenorakel.de>"
-	"\n",
+	"Richard Laager <rlaager@users.sf.net>",
 
 	/** homepage */
 	"http://gaim-irchelper.sf.net",
@@ -214,7 +219,11 @@ static GaimConversation *get_conversation(GaimAccount *account)
 	GaimConversation *conv;
 
 	conv = g_new0(GaimConversation, 1);
+#if GAIM_MAJOR_VERSION > 1
+	conv->type = GAIM_CONV_TYPE_IM;
+#else
 	conv->type = GAIM_CONV_IM;
+#endif
 	gaim_conversation_set_account(conv, account);
 
 	return conv;
@@ -272,7 +281,7 @@ static gboolean auth_timeout(gpointer proto_data)
 static void authserv_identify(const char *command, GaimConnection *connection, IRCHelperStateFlags state)
 {
 	GaimAccount *account;
-	gchar **userparts;
+	gchar **userparts = NULL;
 	const gchar *username;
 	const gchar *password;
 
@@ -280,8 +289,13 @@ static void authserv_identify(const char *command, GaimConnection *connection, I
 
 	account = gaim_connection_get_account(connection);
 
-	userparts = g_strsplit(gaim_account_get_username(account), "@", 2);
-	username = userparts[0];
+	username = gaim_account_get_string(account, PLUGIN_ID "_authname", "");
+	if (NULL == username || '\0' == *username)
+	{
+		userparts = g_strsplit(gaim_account_get_username(account), "@", 2);
+		username = userparts[0];
+	}
+
 	password = gaim_account_get_string(account, PLUGIN_ID "_nickpassword", "");
 
 	if (NULL != username && '\0' != *username &&
@@ -398,7 +412,7 @@ static void jeux_identify(GaimConnection *connection, IRCHelperStateFlags state)
 	}
 
  }
- 
+
 
 /*****************************************************************************
  * NickServ Helper Functions                                                 *
@@ -601,6 +615,151 @@ static void signed_on_cb(GaimConnection *connection)
 	oper_identify(account);
 }
 
+static void conversation_created_cb(GaimConversation *conv)
+{
+	gaim_conversation_set_data(conv, PLUGIN_ID "_start_time",
+	                           GINT_TO_POINTER((int)time(NULL)));
+}
+
+static gboolean writing_chat_msg_cb(GaimAccount *account, const char *who,
+                                  char **message, GaimConversation *conv,
+                                  GaimMessageFlags flags)
+{
+	const gchar *name;
+	const gchar *topic;
+
+	if (!g_str_equal(gaim_account_get_protocol_id(account), IRC_PLUGIN_ID))
+		return FALSE;
+
+	if (NULL == *message)
+		return FALSE;
+
+#if GAIM_MAJOR_VERSION >= 2
+	g_return_val_if_fail(gaim_conversation_get_type(conv) == GAIM_CONV_TYPE_CHAT, FALSE);
+#else
+	g_return_val_if_fail(gaim_conversation_get_type(conv) == GAIM_CONV_CHAT, FALSE);
+#endif
+
+	if (flags & GAIM_MESSAGE_SYSTEM &&
+	    (g_str_has_prefix(*message, MESSAGE_MODE_NOTICE_PREFIX "v ") ||
+	     g_str_has_prefix(*message, MESSAGE_MODE_NOTICE_PREFIX "o ")))
+	{
+		const char *tmp = *message + sizeof(MESSAGE_MODE_NOTICE_PREFIX "X ") - 1;
+		const char *name = gaim_connection_get_display_name(gaim_account_get_connection(account));
+
+		if (g_str_has_prefix(tmp, name) &&
+		    g_str_has_prefix(tmp + strlen(name), MESSAGE_MODE_NOTICE_SUFFIX NICK_CHANSERV))
+		{
+			if (time(NULL) < (time_t)GPOINTER_TO_INT(gaim_conversation_get_data(conv, PLUGIN_ID "_start_time") + 10))
+				return TRUE;
+		}
+	}
+
+	if (flags & GAIM_MESSAGE_SYSTEM &&
+	    (topic = gaim_conv_chat_get_topic(GAIM_CONV_CHAT(conv))) != NULL &&
+	    (name = gaim_conversation_get_name(conv)) != NULL)
+	{
+		char *name_escaped = g_markup_escape_text(name, -1);
+		char *topic_escaped = g_markup_escape_text(topic, -1);
+		char *topic_linkified = gaim_markup_linkify(topic_escaped);
+
+		if (strstr(*message, name_escaped) != NULL &&
+		    strstr(*message, topic_linkified) != NULL)
+		{
+			/* We've got a topic notice. */
+			GaimBlistNode *node;
+
+			if ((node = (GaimBlistNode *)gaim_blist_find_chat(account, name)) != NULL)
+			{
+				const char *last_topic = gaim_blist_node_get_string(node, PLUGIN_ID "_topic");
+
+				/* If we saw this the last time we joined, suppress it. */
+				if (last_topic != NULL && strcmp(topic, last_topic) == 0)
+				{
+#if GAIM_MAJOR_VERSION < 2
+					g_free(*message);
+					*message = NULL;
+#endif
+
+					g_free(name_escaped);
+					g_free(topic_escaped);
+					g_free(topic_linkified);
+
+					return TRUE;
+				}
+				else
+					gaim_blist_node_set_string(node, PLUGIN_ID "_topic", topic);
+			}
+		}
+		g_free(name_escaped);
+		g_free(topic_escaped);
+		g_free(topic_linkified);
+	}
+
+	return FALSE;
+}
+
+static GSList *auto_responses = NULL;
+
+struct auto_response {
+	GaimConnection *gc;
+	char *name;
+	time_t received;
+	char *message;
+};
+
+static gboolean
+expire_auto_responses(gpointer data)
+{
+	GSList *tmp, *cur;
+	struct auto_response *ar;
+
+	tmp = auto_responses;
+
+	while (tmp) {
+		cur = tmp;
+		tmp = tmp->next;
+		ar = (struct auto_response *)cur->data;
+
+		if ((time(NULL) - ar->received) > AUTO_RESPONSE_INTERVAL) {
+			auto_responses = g_slist_remove(auto_responses, ar);
+
+			g_free(ar->message);
+			g_free(ar);
+		}
+	}
+
+	return FALSE; /* do not run again */
+}
+
+static struct auto_response *
+get_auto_response(GaimConnection *gc, const char *name)
+{
+	GSList *tmp;
+	struct auto_response *ar;
+
+	gaim_timeout_add((AUTO_RESPONSE_INTERVAL + 1) * 1000, expire_auto_responses, NULL);
+
+	tmp = auto_responses;
+
+	while (tmp) {
+		ar = (struct auto_response *)tmp->data;
+
+		if (gc == ar->gc && !strncmp(name, ar->name, sizeof(ar->name)))
+			return ar;
+
+		tmp = tmp->next;
+	}
+
+	ar = (struct auto_response *)g_new0(struct auto_response, 1);
+	ar->name = g_strdup(name);
+	ar->gc = gc;
+	ar->received = 0;
+	auto_responses = g_slist_prepend(auto_responses, ar);
+
+	return ar;
+}
+
 static gboolean receiving_im_msg_cb(GaimAccount *account, gchar **sender, gchar **buffer,
 #if GAIM_MAJOR_VERSION >= 2
                                     GaimConversation *conv,
@@ -624,6 +783,32 @@ static gboolean receiving_im_msg_cb(GaimAccount *account, gchar **sender, gchar
 	g_return_val_if_fail(NULL != connection, FALSE);
 	state = GPOINTER_TO_INT(g_hash_table_lookup(states, connection->proto_data));
 
+	/* SUPPRESS EXTRA AUTO-RESPONSES */
+	if (*flags & GAIM_MESSAGE_AUTO_RESP)
+	{
+		/* The same idea as the code in src/server.c. */
+		struct auto_response *ar = get_auto_response(connection, nick);
+		time_t now = time(NULL);
+
+		if ((now - ar->received) <= AUTO_RESPONSE_INTERVAL &&
+		    !strcmp(msg, ar->message))
+		{
+			/* We've recently received an auto-response
+			 * and it's the same as the one we've just received.
+			 * Drop it!
+			 */
+			ar->received = now;
+			return TRUE;
+		}
+
+		ar->received = now;
+		g_free(ar->message);
+		ar->message = g_strdup(msg);
+
+		/* None of the other rules are for auto-responses. */
+		return FALSE;
+	}
+
 
 	/* SIMPLE SUPPRESSION RULES */
 
@@ -641,6 +826,13 @@ static gboolean receiving_im_msg_cb(GaimAccount *account, gchar **sender, gchar
 		return TRUE;
 	}
 
+	/* Suppress GameSurge message(s) of the day. */
+	if (state & IRC_NETWORK_TYPE_GAMESURGE &&
+	    g_str_equal(nick, NICK_GAMESURGE_GLOBAL))
+	{
+		return TRUE;
+	}
+
 	/* Suppress useless Jeux welcome. */
 	if (g_str_equal(nick, NICK_JEUX_WELCOME))
 	{
@@ -675,7 +867,7 @@ static gboolean receiving_im_msg_cb(GaimAccount *account, gchar **sender, gchar
 	if (g_str_equal(nick, "Z") &&
 	    (g_str_has_prefix(msg, MESSAGE_GAIM_NOTICE_PREFIX MESSAGE_FRENCH_LOGIN) ||
 	    g_str_has_prefix(msg, MESSAGE_GAIM_NOTICE_PREFIX MESSAGE_FRENCH_MOTD)))
-	{ 
+	{
 		return TRUE;
 	}
 
@@ -699,6 +891,14 @@ static gboolean receiving_im_msg_cb(GaimAccount *account, gchar **sender, gchar
 		return TRUE;
 	}
 
+	/* Suppress hostname cloak notification on Freenode. */
+	if (g_str_has_suffix(nick, DOMAIN_SUFFIX_FREENODE) &&
+	    g_str_has_prefix(msg, MESSAGE_GAIM_NOTICE_PREFIX NICK_NICKSERV MESSAGE_NICKSERV_CLOAKED))
+	{
+
+		return TRUE;
+	}
+
 
 	/* SLIGHTLY COMPLICATED SUPPRESSION RULES */
 
@@ -725,9 +925,86 @@ static gboolean receiving_im_msg_cb(GaimAccount *account, gchar **sender, gchar
 
 	/* SERVICE MAGIC */
 
+	/* Display ChanServ Access List Add Notifications in the chat window. */
+	if (g_str_has_prefix(msg, MESSAGE_GAIM_NOTICE_PREFIX MESSAGE_CHANSERV_ACCESS_LIST_ADD) &&
+	    g_str_equal(nick, NICK_CHANSERV))
+	{
+		gchar *tmp;
+		gchar *channel;
+		gchar *level = NULL;
+		gchar *msg2;
+
+		channel = gaim_markup_strip_html(msg + sizeof(MESSAGE_GAIM_NOTICE_PREFIX MESSAGE_CHANSERV_ACCESS_LIST_ADD) - 1);
+		if ((tmp = strchr(channel, ' ')) != NULL)
+		{
+			*tmp = '\0';
+			if (g_str_has_prefix(tmp + 1, MESSAGE_CHANSERV_ACCESS_LIST_ADD_WITH_LEVEL))
+			{
+				/* The + 1 and - 1 are both kept to make it clear how that relates with other code. */
+				level = tmp + 1 + sizeof(MESSAGE_CHANSERV_ACCESS_LIST_ADD_WITH_LEVEL) - 1;
+				if ((tmp = strchr(level, ']')) != NULL)
+					*tmp = '\0';
+			}
+		}
+
+		if (NULL == level)
+			msg2 = g_strdup(_("You have been added to the access list."));
+		else
+			msg2 = g_strdup_printf(_("You have been added to the access list with an access level of %s."), level);
+
+#if GAIM_MAJOR_VERSION >= 2
+		chat = gaim_find_conversation_with_account(GAIM_CONV_TYPE_CHAT, channel, account);
+#else
+		chat = gaim_find_conversation_with_account(channel, account);
+#endif
+
+		if (chat)
+		{
+			gaim_conv_chat_write(GAIM_CONV_CHAT(chat), nick,
+			                     msg2, GAIM_MESSAGE_SYSTEM, time(NULL));
+			g_free(channel);
+			g_free(msg2);
+			return TRUE;
+		}
+
+		g_free(channel);
+		g_free(msg2);
+		return FALSE;
+	}
+
+	/* Display ChanServ Access List Delete Notifications in the chat window. */
+	if (g_str_has_prefix(msg, MESSAGE_GAIM_NOTICE_PREFIX MESSAGE_CHANSERV_ACCESS_LIST_DEL) &&
+	    g_str_equal(nick, NICK_CHANSERV))
+	{
+		gchar *tmp;
+		gchar *channel;
+
+		channel = gaim_markup_strip_html(msg + sizeof(MESSAGE_GAIM_NOTICE_PREFIX MESSAGE_CHANSERV_ACCESS_LIST_DEL) - 1);
+		if ((tmp = strchr(channel, ']')) != NULL)
+			*tmp = '\0';
+
+#if GAIM_MAJOR_VERSION >= 2
+		chat = gaim_find_conversation_with_account(GAIM_CONV_TYPE_CHAT, channel, account);
+#else
+		chat = gaim_find_conversation_with_account(channel, account);
+#endif
+
+		if (chat)
+		{
+			gaim_conv_chat_write(GAIM_CONV_CHAT(chat), nick,
+			                     _("You have been removed from the access list."),
+			                     GAIM_MESSAGE_SYSTEM, time(NULL));
+			g_free(channel);
+			return TRUE;
+		}
+
+		g_free(channel);
+		return FALSE;
+	}
+
 	/* Display ChanServ channel join messages in the chat window. */
-	if (g_str_has_prefix(msg, MESSAGE_GAIM_NOTICE_PREFIX "[#") && (
-	    g_str_equal(nick, NICK_CHANSERV) || g_str_equal(nick, NICK_QUAKENET_L)))
+	if (g_str_has_prefix(msg, MESSAGE_GAIM_NOTICE_PREFIX "[#") &&
+	    (g_str_equal(nick, NICK_CHANSERV) || g_str_equal(nick, NICK_QUAKENET_L)))
 	{
 		gchar *msg2;
 		gchar *msg3;
@@ -750,6 +1027,8 @@ static gboolean receiving_im_msg_cb(GaimAccount *account, gchar **sender, gchar
 		 */
 		if ((msg3 = g_strstr_len(msg3, strlen(msg3), "]")))
 		{
+			GaimBlistNode *node;
+
 			*msg3 = '\0';
 
 			/* Make sure it's safe to increment the pointer */
@@ -761,11 +1040,24 @@ static gboolean receiving_im_msg_cb(GaimAccount *account, gchar **sender, gchar
 
 			msg3 += 2;
 
+			if ((node = (GaimBlistNode *)gaim_blist_find_chat(account, channel)) != NULL)
+			{
+				const char *last_msg = gaim_blist_node_get_string(node, PLUGIN_ID "_chanserv_join_msg");
+
+				/* If we saw this the last time we joined, suppress it. */
+				if (last_msg != NULL && strcmp(msg3, last_msg) == 0)
+				{
+					g_free(msg2);
+					return TRUE;
+				}
+				else
+					gaim_blist_node_set_string(node, PLUGIN_ID "_chanserv_join_msg", msg3);
+			}
+
 			/* Write the message to the chat as a system message. */
 #if GAIM_MAJOR_VERSION >= 2
-			chat = gaim_find_conversation_with_account(GAIM_CONV_CHAT, channel, account);
+			chat = gaim_find_conversation_with_account(GAIM_CONV_TYPE_CHAT, channel, account);
 #else
-
 			chat = gaim_find_conversation_with_account(channel, account);
 #endif
 
@@ -905,12 +1197,15 @@ static gboolean plugin_load(GaimPlugin *plugin)
 
 	/* Register protocol preferences. */
 
+	option = gaim_account_option_string_new(_("Auth name"), PLUGIN_ID "_authname", "");
+
+	prpl_info->protocol_options = g_list_append(prpl_info->protocol_options, option);
+
 	option = gaim_account_option_string_new(_("Nick password"), PLUGIN_ID "_nickpassword", "");
 
 	/* My patch to add this functionality was accepted for 2.0.0 and 1.2.0.
 	 * See http://sf.net/support/tracker.php?aid=1108846 */
 #if GAIM_MAJOR_VERSION >= 2 || GAIM_MINOR_VERSION >= 2
-
 	gaim_account_option_set_masked(option, TRUE);
 #endif
 
@@ -925,7 +1220,6 @@ static gboolean plugin_load(GaimPlugin *plugin)
 	/* My patch to add this functionality was accepted for 2.0.0 and 1.2.0.
 	 * See http://sf.net/support/tracker.php?aid=1108846 */
 #if GAIM_MAJOR_VERSION >= 2 || GAIM_MINOR_VERSION >= 2
-
 	gaim_account_option_set_masked(option, TRUE);
 #endif
 
@@ -939,9 +1233,12 @@ static gboolean plugin_load(GaimPlugin *plugin)
 
 	gaim_signal_connect(conn_handle, "signed-on",
 	                    plugin, GAIM_CALLBACK(signed_on_cb), NULL);
+	gaim_signal_connect(conv_handle, "conversation-created",
+	                    plugin, GAIM_CALLBACK(conversation_created_cb), NULL);
 	gaim_signal_connect(conv_handle, "receiving-im-msg",
 	                    plugin, GAIM_CALLBACK(receiving_im_msg_cb), NULL);
-
+	gaim_signal_connect(conv_handle, "writing-chat-msg",
+	                    plugin, GAIM_CALLBACK(writing_chat_msg_cb), NULL);
 
 	return TRUE;
 }
